@@ -1,17 +1,19 @@
 # Copyright 2021, New York University and the TUF contributors
 # SPDX-License-Identifier: MIT OR Apache-2.0
 
-"""Provides an implementation of ``FetcherInterface`` using the Requests
-  HTTP library.
+"""Provides an implementation of ``FetcherInterface`` using the Requests HTTP library.
 """
 
+# requests_fetcher is public but comes from _internal for now (because
+# sigstore-python 1.0 still uses the module from there). requests_fetcher
+# can be moved out of _internal once sigstore-python 1.0 is not relevant.
+
 import logging
-from typing import Dict, Iterator
+from typing import Dict, Iterator, Tuple
 from urllib import parse
 
 # Imports
 import requests
-import requests.exceptions
 
 import tuf
 from tuf.api import exceptions
@@ -25,8 +27,10 @@ class RequestsFetcher(FetcherInterface):
     """An implementation of ``FetcherInterface`` based on the requests library.
 
     Attributes:
-        _sessions: Dictionary of ``Requests.Session`` objects storing a separate
-            session per scheme+hostname combination.
+        socket_timeout: Timeout in seconds, used for both initial connection
+            delay and the maximum delay between bytes received. Default is
+            4 seconds.
+        chunk_size: Chunk size in bytes used when downloading.
     """
 
     def __init__(self) -> None:
@@ -44,14 +48,14 @@ class RequestsFetcher(FetcherInterface):
         # improve efficiency, but avoiding sharing state between different
         # hosts-scheme combinations to minimize subtle security issues.
         # Some cookies may not be HTTP-safe.
-        self._sessions: Dict[str, requests.Session] = {}
+        self._sessions: Dict[Tuple[str, str], requests.Session] = {}
 
         # Default settings
         self.socket_timeout: int = 4  # seconds
         self.chunk_size: int = 400000  # bytes
 
     def _fetch(self, url: str) -> Iterator[bytes]:
-        """Fetches the contents of HTTP/HTTPS url from a remote server
+        """Fetch the contents of HTTP/HTTPS url from a remote server.
 
         Args:
             url: URL string that represents a file location.
@@ -109,8 +113,7 @@ class RequestsFetcher(FetcherInterface):
             response.close()
 
     def _get_session(self, url: str) -> requests.Session:
-        """Returns a different customized requests.Session per schema+hostname
-        combination.
+        """Return a different customized requests.Session per schema+hostname combination.
 
         Raises:
             exceptions.DownloadError: When there is a problem parsing the url.
@@ -119,10 +122,10 @@ class RequestsFetcher(FetcherInterface):
         # reuse connections while minimizing subtle security issues.
         parsed_url = parse.urlparse(url)
 
-        if not parsed_url.scheme or not parsed_url.hostname:
+        if not parsed_url.scheme:
             raise exceptions.DownloadError(f"Failed to parse URL {url}")
 
-        session_index = f"{parsed_url.scheme}+{parsed_url.hostname}"
+        session_index = (parsed_url.scheme, parsed_url.hostname or "")
         session = self._sessions.get(session_index)
 
         if not session:
