@@ -1,7 +1,7 @@
 # Copyright the TUF contributors
 # SPDX-License-Identifier: MIT OR Apache-2.0
 
-"""Trusted collection of client-side TUF Metadata
+"""Trusted collection of client-side TUF Metadata.
 
 ``TrustedMetadataSet`` keeps track of the current valid set of metadata for the
 client, and handles almost every step of the "Detailed client workflow" (
@@ -59,9 +59,9 @@ Example of loading root, timestamp and snapshot:
 >>>         trusted_set.update_snapshot(f.read())
 """
 
+import datetime
 import logging
 from collections import abc
-from datetime import datetime
 from typing import Dict, Iterator, Optional
 
 from tuf.api import exceptions
@@ -71,7 +71,7 @@ logger = logging.getLogger(__name__)
 
 
 class TrustedMetadataSet(abc.Mapping):
-    """Internal class to keep track of trusted metadata in ``Updater``
+    """Internal class to keep track of trusted metadata in ``Updater``.
 
     ``TrustedMetadataSet`` ensures that the collection of metadata in it is valid
     and trusted through the whole client update workflow. It provides easy ways
@@ -79,7 +79,7 @@ class TrustedMetadataSet(abc.Mapping):
     """
 
     def __init__(self, root_data: bytes):
-        """Initialize ``TrustedMetadataSet`` by loading trusted root metadata
+        """Initialize ``TrustedMetadataSet`` by loading trusted root metadata.
 
         Args:
             root_data: Trusted root metadata as bytes. Note that this metadata
@@ -91,7 +91,7 @@ class TrustedMetadataSet(abc.Mapping):
                 error type and content will contain more details.
         """
         self._trusted_set: Dict[str, Metadata] = {}
-        self.reference_time = datetime.utcnow()
+        self.reference_time = datetime.datetime.utcnow()
 
         # Load and validate the local root metadata. Valid initial trusted root
         # metadata is required
@@ -99,41 +99,41 @@ class TrustedMetadataSet(abc.Mapping):
         self._load_trusted_root(root_data)
 
     def __getitem__(self, role: str) -> Metadata:
-        """Returns current ``Metadata`` for ``role``"""
+        """Return current ``Metadata`` for ``role``."""
         return self._trusted_set[role]
 
     def __len__(self) -> int:
-        """Returns number of ``Metadata`` objects in ``TrustedMetadataSet``"""
+        """Return number of ``Metadata`` objects in ``TrustedMetadataSet``."""
         return len(self._trusted_set)
 
     def __iter__(self) -> Iterator[Metadata]:
-        """Returns iterator over ``Metadata`` objects in ``TrustedMetadataSet``"""
+        """Return iterator over ``Metadata`` objects in ``TrustedMetadataSet``."""
         return iter(self._trusted_set.values())
 
     # Helper properties for top level metadata
     @property
     def root(self) -> Metadata[Root]:
-        """Current root ``Metadata``"""
+        """Get current root ``Metadata``."""
         return self._trusted_set[Root.type]
 
     @property
-    def timestamp(self) -> Optional[Metadata[Timestamp]]:
-        """Current timestamp ``Metadata`` or ``None``"""
-        return self._trusted_set.get(Timestamp.type)
+    def timestamp(self) -> Metadata[Timestamp]:
+        """Get current timestamp ``Metadata``."""
+        return self._trusted_set[Timestamp.type]
 
     @property
-    def snapshot(self) -> Optional[Metadata[Snapshot]]:
-        """Current snapshot ``Metadata`` or ``None``"""
-        return self._trusted_set.get(Snapshot.type)
+    def snapshot(self) -> Metadata[Snapshot]:
+        """Get current snapshot ``Metadata``."""
+        return self._trusted_set[Snapshot.type]
 
     @property
-    def targets(self) -> Optional[Metadata[Targets]]:
-        """Current targets ``Metadata`` or ``None``"""
-        return self._trusted_set.get(Targets.type)
+    def targets(self) -> Metadata[Targets]:
+        """Get current top-level targets ``Metadata``."""
+        return self._trusted_set[Targets.type]
 
     # Methods for updating metadata
     def update_root(self, data: bytes) -> Metadata[Root]:
-        """Verifies and loads ``data`` as new root metadata.
+        """Verify and load ``data`` as new root metadata.
 
         Note that an expired intermediate root is considered valid: expiry is
         only checked for the final root in ``update_timestamp()``.
@@ -149,7 +149,7 @@ class TrustedMetadataSet(abc.Mapping):
         Returns:
             Deserialized and verified root ``Metadata`` object
         """
-        if self.timestamp is not None:
+        if Timestamp.type in self._trusted_set:
             raise RuntimeError("Cannot update root after timestamp")
         logger.debug("Updating root")
 
@@ -173,12 +173,12 @@ class TrustedMetadataSet(abc.Mapping):
         new_root.verify_delegate(Root.type, new_root)
 
         self._trusted_set[Root.type] = new_root
-        logger.info("Updated root v%d", new_root.signed.version)
+        logger.debug("Updated root v%d", new_root.signed.version)
 
         return new_root
 
     def update_timestamp(self, data: bytes) -> Metadata[Timestamp]:
-        """Verifies and loads ``data`` as new timestamp metadata.
+        """Verify and load ``data`` as new timestamp metadata.
 
         Note that an intermediate timestamp is allowed to be expired:
         ``TrustedMetadataSet`` will throw an ``ExpiredMetadataError`` in
@@ -199,7 +199,7 @@ class TrustedMetadataSet(abc.Mapping):
         Returns:
             Deserialized and verified timestamp ``Metadata`` object
         """
-        if self.snapshot is not None:
+        if Snapshot.type in self._trusted_set:
             raise RuntimeError("Cannot update timestamp after snapshot")
 
         # client workflow 5.3.10: Make sure final root is not expired.
@@ -219,13 +219,17 @@ class TrustedMetadataSet(abc.Mapping):
 
         # If an existing trusted timestamp is updated,
         # check for a rollback attack
-        if self.timestamp is not None:
+        if Timestamp.type in self._trusted_set:
             # Prevent rolling back timestamp version
             if new_timestamp.signed.version < self.timestamp.signed.version:
                 raise exceptions.BadVersionNumberError(
                     f"New timestamp version {new_timestamp.signed.version} must"
                     f" be >= {self.timestamp.signed.version}"
                 )
+            # Keep using old timestamp if versions are equal.
+            if new_timestamp.signed.version == self.timestamp.signed.version:
+                raise exceptions.EqualVersionNumberError()
+
             # Prevent rolling back snapshot version
             snapshot_meta = self.timestamp.signed.snapshot_meta
             new_snapshot_meta = new_timestamp.signed.snapshot_meta
@@ -239,7 +243,7 @@ class TrustedMetadataSet(abc.Mapping):
         # protection of new timestamp: expiry is checked in update_snapshot()
 
         self._trusted_set[Timestamp.type] = new_timestamp
-        logger.info("Updated timestamp v%d", new_timestamp.signed.version)
+        logger.debug("Updated timestamp v%d", new_timestamp.signed.version)
 
         # timestamp is loaded: raise if it is not valid _final_ timestamp
         self._check_final_timestamp()
@@ -247,16 +251,15 @@ class TrustedMetadataSet(abc.Mapping):
         return new_timestamp
 
     def _check_final_timestamp(self) -> None:
-        """Raise if timestamp is expired"""
+        """Raise if timestamp is expired."""
 
-        assert self.timestamp is not None  # nosec
         if self.timestamp.signed.is_expired(self.reference_time):
             raise exceptions.ExpiredMetadataError("timestamp.json is expired")
 
     def update_snapshot(
         self, data: bytes, trusted: Optional[bool] = False
     ) -> Metadata[Snapshot]:
-        """Verifies and loads ``data`` as new snapshot metadata.
+        """Verify and load ``data`` as new snapshot metadata.
 
         Note that an intermediate snapshot is allowed to be expired and version
         is allowed to not match timestamp meta version: ``TrustedMetadataSet``
@@ -284,9 +287,9 @@ class TrustedMetadataSet(abc.Mapping):
             Deserialized and verified snapshot ``Metadata`` object
         """
 
-        if self.timestamp is None:
+        if Timestamp.type not in self._trusted_set:
             raise RuntimeError("Cannot update snapshot before timestamp")
-        if self.targets is not None:
+        if Targets.type in self._trusted_set:
             raise RuntimeError("Cannot update snapshot after targets")
         logger.debug("Updating snapshot")
 
@@ -313,7 +316,7 @@ class TrustedMetadataSet(abc.Mapping):
         # used in rollback protection: it is checked when targets is updated
 
         # If an existing trusted snapshot is updated, check for rollback attack
-        if self.snapshot is not None:
+        if Snapshot.type in self._trusted_set:
             for filename, fileinfo in self.snapshot.signed.meta.items():
                 new_fileinfo = new_snapshot.signed.meta.get(filename)
 
@@ -334,7 +337,7 @@ class TrustedMetadataSet(abc.Mapping):
         # protection of new snapshot: it is checked when targets is updated
 
         self._trusted_set[Snapshot.type] = new_snapshot
-        logger.info("Updated snapshot v%d", new_snapshot.signed.version)
+        logger.debug("Updated snapshot v%d", new_snapshot.signed.version)
 
         # snapshot is loaded, but we raise if it's not valid _final_ snapshot
         self._check_final_snapshot()
@@ -342,10 +345,8 @@ class TrustedMetadataSet(abc.Mapping):
         return new_snapshot
 
     def _check_final_snapshot(self) -> None:
-        """Raise if snapshot is expired or meta version does not match"""
+        """Raise if snapshot is expired or meta version does not match."""
 
-        assert self.snapshot is not None  # nosec
-        assert self.timestamp is not None  # nosec
         if self.snapshot.signed.is_expired(self.reference_time):
             raise exceptions.ExpiredMetadataError("snapshot.json is expired")
         snapshot_meta = self.timestamp.signed.snapshot_meta
@@ -356,7 +357,7 @@ class TrustedMetadataSet(abc.Mapping):
             )
 
     def update_targets(self, data: bytes) -> Metadata[Targets]:
-        """Verifies and loads ``data`` as new top-level targets metadata.
+        """Verify and load ``data`` as new top-level targets metadata.
 
         Args:
             data: Unverified new targets metadata as bytes
@@ -373,7 +374,7 @@ class TrustedMetadataSet(abc.Mapping):
     def update_delegated_targets(
         self, data: bytes, role_name: str, delegator_name: str
     ) -> Metadata[Targets]:
-        """Verifies and loads ``data`` as new metadata for target ``role_name``.
+        """Verify and load ``data`` as new metadata for target ``role_name``.
 
         Args:
             data: Unverified new metadata as bytes
@@ -388,7 +389,7 @@ class TrustedMetadataSet(abc.Mapping):
         Returns:
             Deserialized and verified targets ``Metadata`` object
         """
-        if self.snapshot is None:
+        if Snapshot.type not in self._trusted_set:
             raise RuntimeError("Cannot load targets before snapshot")
 
         # Targets cannot be loaded if final snapshot is expired or its version
@@ -429,12 +430,12 @@ class TrustedMetadataSet(abc.Mapping):
             raise exceptions.ExpiredMetadataError(f"New {role_name} is expired")
 
         self._trusted_set[role_name] = new_delegate
-        logger.info("Updated %s v%d", role_name, version)
+        logger.debug("Updated %s v%d", role_name, version)
 
         return new_delegate
 
     def _load_trusted_root(self, data: bytes) -> None:
-        """Verifies and loads ``data`` as trusted root metadata.
+        """Verify and load ``data`` as trusted root metadata.
 
         Note that an expired initial root is considered valid: expiry is
         only checked for the final root in ``update_timestamp()``.
@@ -449,4 +450,4 @@ class TrustedMetadataSet(abc.Mapping):
         new_root.verify_delegate(Root.type, new_root)
 
         self._trusted_set[Root.type] = new_root
-        logger.info("Loaded trusted root v%d", new_root.signed.version)
+        logger.debug("Loaded trusted root v%d", new_root.signed.version)
