@@ -1,7 +1,7 @@
 # Copyright 2021, New York University and the TUF contributors
 # SPDX-License-Identifier: MIT OR Apache-2.0
 
-"""Configuration options for ``Updater`` class
+"""Configuration options for ``Updater`` class.
 """
 
 from dataclasses import dataclass
