@@ -17,19 +17,20 @@ logger = logging.getLogger(__name__)
 
 
 # Classes
-class FetcherInterface:
+class FetcherInterface(metaclass=abc.ABCMeta):
     """Defines an interface for abstract network download.
 
     By providing a concrete implementation of the abstract interface,
     users of the framework can plug-in their preferred/customized
     network stack.
-    """
 
-    __metaclass__ = abc.ABCMeta
+    Implementations of FetcherInterface only need to implement ``_fetch()``.
+    The public API of the class is already implemented.
+    """
 
     @abc.abstractmethod
     def _fetch(self, url: str) -> Iterator[bytes]:
-        """Fetches the contents of HTTP/HTTPS ``url`` from a remote server.
+        """Fetch the contents of HTTP/HTTPS ``url`` from a remote server.
 
         Implementations must raise ``DownloadHTTPError`` if they receive
         an HTTP error code.
@@ -50,7 +51,7 @@ class FetcherInterface:
         raise NotImplementedError  # pragma: no cover
 
     def fetch(self, url: str) -> Iterator[bytes]:
-        """Fetches the contents of HTTP/HTTPS ``url`` from a remote server.
+        """Fetch the contents of HTTP/HTTPS ``url`` from a remote server.
 
         Args:
             url: URL string that represents a file location.
