@@ -1,7 +1,7 @@
 # Copyright 2020, New York University and the TUF contributors
 # SPDX-License-Identifier: MIT OR Apache-2.0
 
-"""Client update workflow implementation
+"""Client update workflow implementation.
 
 The ``Updater`` class provides an implementation of the
 `TUF client workflow
@@ -28,6 +28,10 @@ High-level description of ``Updater`` functionality:
       * ``Updater.download_target()`` downloads a target file and ensures it is
         verified correct by the metadata.
 
+Note that applications using ``Updater`` should be 'single instance'
+applications: running multiple instances that use the same cache directories at
+the same time is not supported.
+
 A simple example of using the Updater to implement a Python TUF client that
 downloads target files is available in `examples/client_example
 <https://github.com/theupdateframework/python-tuf/tree/develop/examples/client_example>`_.
@@ -70,6 +74,8 @@ class Updater:
             downloads. Can be individually set in ``download_target()``
         fetcher: ``Optional``; ``FetcherInterface`` implementation used to
             download both metadata and targets. Default is ``RequestsFetcher``
+        config: ``Optional``; ``UpdaterConfig`` could be used to setup common
+            configuration options.
 
     Raises:
         OSError: Local root.json cannot be read
@@ -100,7 +106,7 @@ class Updater:
         self.config = config or UpdaterConfig()
 
     def refresh(self) -> None:
-        """Refreshes top-level metadata.
+        """Refresh top-level metadata.
 
         Downloads, verifies, and loads metadata for the top-level roles in the
         specified order (root -> timestamp -> snapshot -> targets) implementing
@@ -114,8 +120,8 @@ class Updater:
         that happens on demand during ``get_targetinfo()``. However, if the
         repository uses `consistent_snapshot
         <https://theupdateframework.github.io/specification/latest/#consistent-snapshots>`_,
-        then all metadata downloaded downloaded by the Updater will use the same
-        consistent repository state.
+        then all metadata downloaded by the Updater will use the same consistent
+        repository state.
 
         Raises:
             OSError: New metadata could not be written to disk
@@ -137,7 +143,7 @@ class Updater:
         return os.path.join(self.target_dir, filename)
 
     def get_targetinfo(self, target_path: str) -> Optional[TargetFile]:
-        """Returns ``TargetFile`` instance with information for ``target_path``.
+        """Return ``TargetFile`` instance with information for ``target_path``.
 
         The return value can be used as an argument to
         ``download_target()`` and ``find_cached_target()``.
@@ -162,7 +168,7 @@ class Updater:
             ``TargetFile`` instance or ``None``.
         """
 
-        if self._trusted_set.targets is None:
+        if Targets.type not in self._trusted_set:
             self.refresh()
         return self._preorder_depth_first_walk(target_path)
 
@@ -171,7 +177,7 @@ class Updater:
         targetinfo: TargetFile,
         filepath: Optional[str] = None,
     ) -> Optional[str]:
-        """Checks whether a local file is an up to date target
+        """Check whether a local file is an up to date target.
 
         Args:
             targetinfo: ``TargetFile`` from ``get_targetinfo()``.
@@ -183,7 +189,7 @@ class Updater:
 
         Returns:
             Local file path if the file is an up to date target file.
-            None if file is not found or it is not up to date.
+            ``None`` if file is not found or it is not up to date.
         """
 
         if filepath is None:
@@ -202,7 +208,7 @@ class Updater:
         filepath: Optional[str] = None,
         target_base_url: Optional[str] = None,
     ) -> str:
-        """Downloads the target file specified by ``targetinfo``.
+        """Download the target file specified by ``targetinfo``.
 
         Args:
             targetinfo: ``TargetFile`` from ``get_targetinfo()``.
@@ -254,13 +260,13 @@ class Updater:
             with open(filepath, "wb") as destination_file:
                 shutil.copyfileobj(target_file, destination_file)
 
-        logger.info("Downloaded target %s", targetinfo.path)
+        logger.debug("Downloaded target %s", targetinfo.path)
         return filepath
 
     def _download_metadata(
         self, rolename: str, length: int, version: Optional[int] = None
     ) -> bytes:
-        """Download a metadata file and return it as bytes"""
+        """Download a metadata file and return it as bytes."""
         encoded_name = parse.quote(rolename, "")
         if version is None:
             url = f"{self._metadata_base_url}{encoded_name}.json"
@@ -275,6 +281,7 @@ class Updater:
 
     def _persist_metadata(self, rolename: str, data: bytes) -> None:
         """Write metadata to disk atomically to avoid data loss."""
+        temp_file_name: Optional[str] = None
         try:
             # encode the rolename to avoid issues with e.g. path separators
             encoded_name = parse.quote(rolename, "")
@@ -282,14 +289,15 @@ class Updater:
             with tempfile.NamedTemporaryFile(
                 dir=self._dir, delete=False
             ) as temp_file:
+                temp_file_name = temp_file.name
                 temp_file.write(data)
             os.replace(temp_file.name, filename)
         except OSError as e:
             # remove tempfile if we managed to create one,
             # then let the exception happen
-            if temp_file:
+            if temp_file_name is not None:
                 try:
-                    os.remove(temp_file.name)
+                    os.remove(temp_file_name)
                 except FileNotFoundError:
                     pass
             raise e
@@ -322,7 +330,7 @@ class Updater:
                 break
 
     def _load_timestamp(self) -> None:
-        """Load local and remote timestamp metadata"""
+        """Load local and remote timestamp metadata."""
         try:
             data = self._load_local_metadata(Timestamp.type)
             self._trusted_set.update_timestamp(data)
@@ -334,11 +342,17 @@ class Updater:
         data = self._download_metadata(
             Timestamp.type, self.config.timestamp_max_length
         )
-        self._trusted_set.update_timestamp(data)
+        try:
+            self._trusted_set.update_timestamp(data)
+        except exceptions.EqualVersionNumberError:
+            # If the new timestamp version is the same as current, discard the
+            # new timestamp. This is normal and it shouldn't raise any error.
+            return
+
         self._persist_metadata(Timestamp.type, data)
 
     def _load_snapshot(self) -> None:
-        """Load local (and if needed remote) snapshot metadata"""
+        """Load local (and if needed remote) snapshot metadata."""
         try:
             data = self._load_local_metadata(Snapshot.type)
             self._trusted_set.update_snapshot(data, trusted=True)
@@ -347,7 +361,6 @@ class Updater:
             # Local snapshot does not exist or is invalid: update from remote
             logger.debug("Local snapshot not valid as final: %s", e)
 
-            assert self._trusted_set.timestamp is not None  # nosec
             snapshot_meta = self._trusted_set.timestamp.signed.snapshot_meta
             length = snapshot_meta.length or self.config.snapshot_max_length
             version = None
@@ -376,8 +389,13 @@ class Updater:
             # Local 'role' does not exist or is invalid: update from remote
             logger.debug("Failed to load local %s: %s", role, e)
 
-            assert self._trusted_set.snapshot is not None  # nosec
-            metainfo = self._trusted_set.snapshot.signed.meta[f"{role}.json"]
+            snapshot = self._trusted_set.snapshot.signed
+            metainfo = snapshot.meta.get(f"{role}.json")
+            if metainfo is None:
+                raise exceptions.RepositoryError(
+                    f"Role {role} was delegated but is not part of snapshot"
+                )
+
             length = metainfo.length or self.config.targets_max_length
             version = None
             if self._trusted_set.root.signed.consistent_snapshot:
@@ -436,17 +454,17 @@ class Updater:
                 child_roles_to_visit = []
                 # NOTE: This may be a slow operation if there are many
                 # delegated roles.
-                for child_role in targets.delegations.roles.values():
-                    if child_role.is_delegated_path(target_filepath):
-                        logger.debug("Adding child role %s", child_role.name)
+                for (
+                    child_name,
+                    terminating,
+                ) in targets.delegations.get_roles_for_target(target_filepath):
 
-                        child_roles_to_visit.append(
-                            (child_role.name, role_name)
-                        )
-                        if child_role.terminating:
-                            logger.debug("Not backtracking to other roles")
-                            delegations_to_visit = []
-                            break
+                    logger.debug("Adding child role %s", child_name)
+                    child_roles_to_visit.append((child_name, role_name))
+                    if terminating:
+                        logger.debug("Not backtracking to other roles")
+                        delegations_to_visit = []
+                        break
                 # Push 'child_roles_to_visit' in reverse order of appearance
                 # onto 'delegations_to_visit'.  Roles are popped from the end of
                 # the list.
@@ -465,5 +483,5 @@ class Updater:
 
 
 def _ensure_trailing_slash(url: str) -> str:
-    """Return url guaranteed to end in a slash"""
+    """Return url guaranteed to end in a slash."""
     return url if url.endswith("/") else f"{url}/"
